package rest;

import java.math.BigDecimal;
import java.net.URI;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import sklep.db.DBConnection;
import sklep.db.DBException;
import sklep.db.ProductDAO;
import sklep.db.RecordNotFound;
import sklep.model.Product;
import sklep.model.ProductList;
import sklep.photo.PhotoUtil;

// JAX-RS - część Javy EE
// Aplikacja (projekt) składa się z wielu "resource classes", z których każda obsługuje pewien rodzaj encji
// np. pod adresem /products działa ta klasa RProduct, która zajmuje się obsługą produktów
// a pod adresem /orders działa ROrder, która zajmuje się zamówieniami.
// Przykładowa dobra (szczegółowa) dokumentacja: https://eclipse-ee4j.github.io/jersey.github.io/documentation/latest3x/index.html

@Path("/products")
public class RProduct {
	// Gdy w Produces jest wiele formatów, to klient może wybrać za pomocą nagłówka Accept
	// Gdy w Consumes jest wiele formatów, to klient może przysłać dane w dowolnym z nich (nagłówek Content-Type)
	@GET
	@Produces({"application/json", "application/xml", "application/pdf", "text/plain;charset=UTF-8"})
	public ProductList readAllProducts() throws DBException {
		try(DBConnection db = DBConnection.open()) {
			ProductDAO productDAO = db.productDAO();
			return new ProductList(productDAO.readAll());
		}
	}
	
	// Może też być tak, że kilka metod działa pod tym samym adresem, ale służą one do tworzenia odpowiedzi w różnych formatach.
	// Przykład: tworzenie HTML w oddzielnej metodzie
	@GET
	@Produces("text/html;charset=UTF-8")
	public String readAllProductsHTML() throws DBException {
		try(DBConnection db = DBConnection.open()) {
			ProductDAO productDAO = db.productDAO();
			List<Product> products = productDAO.readAll();
			StringBuilder txt = new StringBuilder("<!DOCTYPE html>\n<html><body>\n");
			txt.append("<h1>Lista produktów</h1>\n");
			for(Product product : products) {
				txt.append(product.toHtml()).append('\n');
			}
			txt.append("</body></html>");
			return txt.toString();
		}
	}

	@Path("/{id}")
	@GET
	@Produces({"application/json", "application/xml", "application/pdf", "text/plain"})
	public Product readOneProduct(@PathParam("id") int productId) throws DBException, RecordNotFound {
		try(DBConnection db = DBConnection.open()) {
			ProductDAO productDAO = db.productDAO();
			return productDAO.findById(productId);
		}
	}
	
	@Path("/{id}")
	@GET
	@Produces("text/html;charset=UTF-8")
	public String readOneProductsHTML(@PathParam("id") int productId) throws DBException, RecordNotFound {
		Product product = readOneProduct(productId);		
		return "<!DOCTYPE html>\n<html><body>" + product.toHtml() + "</body></html>";
	}
	
	@Path("/{id}/foto")
	@GET
	@Produces("image/jpeg")
	public byte[] foto(@PathParam("id") int productId) throws DBException, RecordNotFound {
		return PhotoUtil.readBytes(productId);
	}
	
	@Path("/{id}/price")
	@GET
	@Produces({"application/json", "text/plain"})
	public BigDecimal getPrice(@PathParam("id") int productId) throws DBException, RecordNotFound {
		try(DBConnection db = DBConnection.open()) {
			ProductDAO productDAO = db.productDAO();
			Product product = productDAO.findById(productId);
			return product.getPrice();
		}
	}
	
	// W metodach HTTP POST i PUT klient przysyła dane na serwer (tzw. body / entity / content).
	// Adnotacja @Consumes mówi w jakim formacie powinny być te dane.
	// Metoda w JAX-RS może posiadać co najwyżej jeden parametr nieoznaczony żadną adnotacją.
	// Właśnie poprzez ten parametr przekazywane są dane przysłane w treści zapytania (w praktyce: POST i PUT).
	@Path("/{id}/price")
	@PUT
	@Consumes({"application/json", "text/plain"})
	public void setPrice(@PathParam("id") int productId, BigDecimal newPrice) throws DBException, RecordNotFound {
		try(DBConnection db = DBConnection.open()) {
			ProductDAO productDAO = db.productDAO();
			Product product = productDAO.findById(productId);
			product.setPrice(newPrice);
			productDAO.save(product);
			db.commit();
		}
	}
	
	@Path("/{id}")
	@PUT
	@Consumes({"application/json", "application/xml"})
	public void updateProduct(@PathParam("id") int productId, Product product) throws DBException {
		try(DBConnection db = DBConnection.open()) {
			ProductDAO productDAO = db.productDAO();
			if(product.getProductId() == null) {
				// jeśli przysłany rekord nie posiada ID, to utawiam ID z adresu
				// w szczególności pozwalam na zapisanie nowego produktu, ale pod określonym adresem
				product.setProductId(productId);
			} else if(product.getProductId() != productId) {
				// ale jeśli ID było ustawione, ale było inne niż w adresie, to jest to podejrzana sytuacja i takie zapytania nie wykonamy
				throw new IllegalArgumentException("ID produktu nie zgodne z adresem URL");
			}
			productDAO.save(product);
			db.commit();
		}
	}
	
	// za pomocą POST będzie można przysłać wyłącznie nowe produkty be określonego ID,
	// a aplikacja sama wybierze nowe ID z sekwencjii
	// w wyniku zostanie odesłany uzupełniony produkt
	@POST
	@Consumes({"application/json", "application/xml"})
	public Response addProduct(Product product,
			@Context UriInfo uriInfo) throws DBException {
		try(DBConnection db = DBConnection.open()) {
			ProductDAO productDAO = db.productDAO();
			if(product.getProductId() != null) {
				throw new IllegalArgumentException("Nowy produkt nie może mieć ustawionego ID");
			}
			productDAO.insertNew(product);
			db.commit();
			URI uri = uriInfo.getAbsolutePathBuilder().path("/{id}").build(product.getProductId());
			return Response.created(uri).build();
		}
	}
	
	@DELETE
	@Path("/{id}")
	public void removeProduct(@PathParam("id") int productId) throws DBException, RecordNotFound {
		try(DBConnection db = DBConnection.open()) {
			ProductDAO productDAO = db.productDAO();
			if(productDAO.delete(productId)) {
				db.commit();
			} else {
				throw new RecordNotFound("Nie ma produktu nr " + productId);
			}
		}
	}
}
